@@ -26,6 +26,7 @@ import scipy.constants as codata
 from wofryimpl.propagator.util.undulator_coherent_mode_decomposition_1d import UndulatorCoherentModeDecomposition1D
 from syned.storage_ring.electron_beam import ElectronBeam
 from syned.storage_ring.magnetic_structures.undulator import Undulator
+from orangecontrib.esrf.wofry.util.light_source import WOLightSourceCMD
 
 
 class OWUndulatorCoherentModeDecomposition1D(WofryWidget):
@@ -89,10 +90,10 @@ class OWUndulatorCoherentModeDecomposition1D(WofryWidget):
 
     def __init__(self):
 
-        super().__init__(is_automatic=False, show_view_options=True, show_script_tab=False)
+        super().__init__(is_automatic=False, show_view_options=True, show_script_tab=True)
 
         self.runaction = widget.OWAction("Generate Wavefront", self)
-        self.runaction.triggered.connect(self.calculate)
+        self.runaction.triggered.connect(self.calculate_and_send_mode)
         self.addAction(self.runaction)
 
 
@@ -101,7 +102,7 @@ class OWUndulatorCoherentModeDecomposition1D(WofryWidget):
 
         button_box = oasysgui.widgetBox(self.controlArea, "", addSpace=False, orientation="horizontal")
 
-        button = gui.button(button_box, self, "Calculate", callback=self.calculate)
+        button = gui.button(button_box, self, "Calculate and Send mode", callback=self.calculate_and_send_mode)
         font = QFont(button.font())
         font.setBold(True)
         button.setFont(font)
@@ -182,12 +183,13 @@ class OWUndulatorCoherentModeDecomposition1D(WofryWidget):
                      callback=self.set_visible,
                      sendSelectedValue=False, orientation="horizontal")
 
+        left_box_3 = oasysgui.widgetBox(self.tab_settings, "Send Wavefront", addSpace=True, orientation="vertical")
         self.mode_index_box = oasysgui.widgetBox(left_box_3, "", addSpace=True, orientation="vertical", )
 
         left_box_5 = oasysgui.widgetBox(self.mode_index_box, "", addSpace=True, orientation="horizontal", )
         tmp = oasysgui.lineEdit(left_box_5, self, "mode_index", "Send mode",
                         labelWidth=200, valueType=int, tooltip = "mode_index",
-                        orientation="horizontal")
+                        orientation="horizontal", callback=self.send_mode)
 
         gui.button(left_box_5, self, "+1", callback=self.increase_mode_index, width=30)
         gui.button(left_box_5, self, "-1", callback=self.decrease_mode_index, width=30)
@@ -372,15 +374,18 @@ class OWUndulatorCoherentModeDecomposition1D(WofryWidget):
                 else:
                     setattr(self, variable_name, variable_value)
 
-                self.generate()
+                self.send_mode()
 
     def get_light_source(self):
-        return WOLightSource()
-
+        return WOLightSourceCMD(name="name", undulator_coherent_mode_decomposition_1d=self.coherent_mode_decomposition)
 
     def generate(self):
         pass
 
+    def calculate_and_send_mode(self):
+        self.calculate()
+        self.send_mode()
+
     def calculate(self):
 
         self.wofry_output.setText("")
@@ -429,68 +434,25 @@ class OWUndulatorCoherentModeDecomposition1D(WofryWidget):
         else:
             self.progressBarFinished()
 
-
-        self.send_mode()
-
-    def send_mode(self):
-
-        beamline = WOBeamline(light_source=self.get_light_source())
         try:
+            beamline = WOBeamline(light_source=self.get_light_source())
             self.wofry_python_script.set_code(beamline.to_python_code())
         except:
             pass
 
+        # self.send_mode()
+
+    def send_mode(self):
+
+        if self.coherent_mode_decomposition is None:
+            self.calculate()
+
+        beamline = WOBeamline(light_source=self.get_light_source())
+        print(">>> sending mode: ", int(self.mode_index))
         self.send("WofryData", WofryData(
-            wavefront=self.coherent_mode_decomposition.get_eigenvector_wavefront(self.mode_index),
+            wavefront=self.coherent_mode_decomposition.get_eigenvector_wavefront(int(self.mode_index)),
             beamline=beamline))
 
-
-    def generate_python_code(self):
-
-        if self.scan_direction == 0:
-            scan_direction = "H"
-            sigmaxx=self.sigma_h
-            sigmaxpxp=self.sigma_divergence_h
-        else:
-            scan_direction = "V"
-            sigmaxx=self.sigma_v
-            sigmaxpxp=self.sigma_divergence_v
-        if self.flag_gsm == 0:
-            useGSMapproximation = 0
-        elif self.flag_gsm == 1:
-            useGSMapproximation = 1
-
-
-        txt = "#"
-        txt += "\n# create input_wavefront\n#"
-        txt += "\n#"
-        txt += "\nfrom wofryimpl.propagator.util.undulator_coherent_mode_decomposition_1d import UndulatorCoherentModeDecomposition1D"
-
-        txt += "\n# main calculation"
-        txt += "\ncoherent_mode_decomposition = UndulatorCoherentModeDecomposition1D("
-        txt += "\n    electron_energy=%g," % self.electron_energy_in_GeV
-        txt += "\n    electron_current=%g," % self.ring_current
-        txt += "\n    undulator_period=%g," % self.period_length
-        txt += "\n    undulator_nperiods=%g," % self.number_of_periods
-        txt += "\n    K=%g" % self.K_vertical
-        txt += "\n    photon_energy=%g," % self.photon_energy
-        txt += "\n    abscissas_interval=%g," % (self.range_to - self.range_from)
-        txt += "\n    number_of_points=%d,",self.number_of_points
-        txt += "\n    distance_to_screen=100.0,"
-        txt += "\n    scan_direction='%s'," % scan_direction
-        txt += "\n    sigmaxx=%g," % sigmaxx
-        txt += "\n    sigmaxpxp=%g," % sigmaxpxp
-        txt += "\n    useGSMapproximation=%d," % useGSMapproximation
-        txt += "\n# make calculation"
-        txt += "\ncoherent_mode_decomposition_results = coherent_mode_decomposition.calculate()"
-        txt += "\ninput_wavefront = coherent_mode_decomposition_results.get_eigenfunction_wavefront(%d)" % self.mode_index
-
-
-        txt += "\n\n\nfrom srxraylib.plot.gol import plot"
-        txt += "\nplot(input_wavefront.get_abscissas(),input_wavefront.get_intensity())"
-
-        return txt
-
     def do_plot_results(self, progressBarValue):
         if not self.coherent_mode_decomposition is None:
 
@@ -499,8 +461,15 @@ class OWUndulatorCoherentModeDecomposition1D(WofryWidget):
             #
             # plot emission size
             #
-            self.plot_data1D(1e6 * self.coherent_mode_decomposition.output_wavefront.get_abscissas(),
-                             self.coherent_mode_decomposition.output_wavefront.get_intensity(),
+            if self.flag_gsm:
+                abscissas = self.coherent_mode_decomposition.abscissas
+                indices = numpy.arange(abscissas.size)
+                intensity = self.coherent_mode_decomposition.CSD[indices,indices]
+            else:
+                abscissas = self.coherent_mode_decomposition.abscissas
+                intensity = self.coherent_mode_decomposition.output_wavefront.get_intensity()
+            self.plot_data1D(1e6 * abscissas,
+                             intensity,
                              progressBarValue=progressBarValue,
                              tabs_canvas_index=0,
                              plot_canvas_index=0,
